@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 from mongoengine import BooleanField
 from mongoengine import DateTimeField
 from mongoengine import Document
@@ -7,6 +9,8 @@ from mongoengine import ListField
 from mongoengine import ReferenceField
 from mongoengine import StringField
 
+from datetime import datetime
+
 
 class User(Document):
     email = StringField(required=True, max_length=50)
@@ -36,6 +40,16 @@ class Post(Document):
     tags = ListField(StringField(max_length=30))
     comments = ListField(EmbeddedDocumentField(Comment))
     pub_date = DateTimeField()
+    created_date = DateTimeField()
     published = BooleanField()
     creator = EmbeddedDocumentField(EmbeddedUser)    # for testing purposes
+    wanted_published = ListField(BooleanField())  # used for testing boolean list
+    published_dates = ListField(DateTimeField())  # used for testing datefield lists
+    past_authors = ListField(ReferenceField(User))  # used for testing reference fields
 
+    def save(self, *args, **kwargs):
+        if not self.created_date:
+            self.created_date = datetime.utcnow()
+        if self.published:
+            self.published_dates.append(datetime.utcnow())
+        super(Post, self).save(*args, **kwargs)
