@@ -1,13 +1,10 @@
+# -*- coding: utf-8 -*-
 """
     TODO move permission checks to the dispatch view thingee
 """
 
-from datetime import datetime
-
 from django.contrib import messages
 from django.core.urlresolvers import reverse
-from django.forms import widgets
-from django.forms.widgets import DateTimeInput, CheckboxInput
 from django.http import HttpResponseForbidden
 from django.views.generic.edit import DeletionMixin
 from django.views.generic import DetailView
@@ -16,14 +13,22 @@ from django.views.generic import TemplateView
 from django.views.generic.edit import FormView
 
 from mongoengine.fields import EmbeddedDocumentField, ListField
-
 from mongonaut.forms import DocumentListForm
 from mongonaut.forms import DocumentDetailForm
 from mongonaut.forms import document_detail_form_factory
+from mongonaut.mixins import MongonautFormViewMixin
 from mongonaut.mixins import MongonautViewMixin
 from mongonaut.utils import is_valid_object_id
 
 
+def setattr_doc(document, attr, value):
+    """Used to set the value of a document.  Ensures an empty value may be given."""
+    if value or isinstance(value, bool):
+        setattr(document, attr, value)
+    else:
+        setattr(document, attr, None)
+
+
 class IndexView(MongonautViewMixin, ListView):
 
     template_name = "mongonaut/index.html"
@@ -83,7 +88,7 @@ class DocumentListView(MongonautViewMixin, FormView):
         # search. move this to get_queryset
         # search. move this to get_queryset
         q = self.request.GET.get('q')
-        query = self.get_qset(queryset, q)
+        self.get_qset(queryset, q)
 
         ### Start pagination
         ### Note:
@@ -216,7 +221,7 @@ class DocumentDetailView(MongonautViewMixin, TemplateView):
         return context
 
 
-class DocumentEditFormView(MongonautViewMixin, FormView):
+class DocumentEditFormView(MongonautViewMixin, FormView, MongonautFormViewMixin):
     """ :args: <app_label> <document_name> <id> """
 
     template_name = "mongonaut/document_edit_form.html"
@@ -254,41 +259,14 @@ class DocumentEditFormView(MongonautViewMixin, FormView):
         self.document = self.document_type.objects.get(id=self.ident)
         self.form = DocumentDetailForm()
 
-        self.form = document_detail_form_factory(form=self.form, document_type=self.document_type, initial=self.document)
         if self.request.method == 'POST':
-            self.form.data = self.request.POST
-            self.form.is_bound = True
-            if self.form.is_valid():
-                for key, field in self.form.fields.items():
-                    # If the field has no value do not attempt to access it.
-                    if not self.request.POST[key]:
-                        continue
-
-                    if 'readonly' in field.widget.attrs:
-                        # For _id or things specified as such
-                        continue
-
-                    if isinstance(field.widget, DateTimeInput):
-                        format = field.widget.format
-                        setattr(self.document, key, datetime.strptime(self.request.POST[key], format))
-                        continue
-
-                    if isinstance(field.widget, widgets.Select):
-                        # supporting reference fields!
-                        value = field.mongofield.document_type.objects.get(id=self.request.POST[key])
-                        setattr(self.document, key, value)
-                        continue
-
-                    # for strings
-                    setattr(self.document, key, self.form.cleaned_data[key])
-
-                self.document.save()
-                messages.add_message(self.request, messages.INFO, 'Your changes have been saved.')
-
+            self.form = self.process_post_form('Your changes have been saved.')
+        else:
+            self.form = document_detail_form_factory(form=self.form, document_type=self.document_type, initial=self.document)
         return self.form
 
 
-class DocumentAddFormView(MongonautViewMixin, FormView):
+class DocumentAddFormView(MongonautViewMixin, FormView, MongonautFormViewMixin):
     """ :args: <app_label> <document_name> <id> """
 
     template_name = "mongonaut/document_add_form.html"
@@ -318,44 +296,11 @@ class DocumentAddFormView(MongonautViewMixin, FormView):
         self.set_mongonaut_base()
         self.document_type = getattr(self.models, self.document_name)
         self.form = DocumentDetailForm()
-        self.form = document_detail_form_factory(form=self.form, document_type=self.document_type)
+
         if self.request.method == 'POST':
-            self.form.data = self.request.POST
-            self.form.is_bound = True
-            if self.form.is_valid():
-                self.document = self.document_type()
-                for key, field in self.form.fields.items():
-                    # If the field has no value do not attempt to access it.
-                    if not self.request.POST[key]:
-                        continue
-
-                    if 'readonly' in field.widget.attrs:
-                        # For _id
-                        continue
-
-                    if isinstance(field.widget, DateTimeInput):
-                        format = field.widget.format
-                        setattr(self.document, key, datetime.strptime(self.request.POST[key], format))
-                        continue
-
-                    if isinstance(field.widget, CheckboxInput):
-                        if key in self.request.POST:
-                            setattr(self.document, key, True)
-                        else:
-                            setattr(self.document, key, False)
-                        continue
-
-                    if isinstance(field.widget, widgets.Select):
-                        # supporting reference fields!
-                        value = field.mongofield.document_type.objects.get(id=self.request.POST[key])
-                        setattr(self.document, key, value)
-                        continue
-
-                    # for strings
-                    setattr(self.document, key, self.request.POST[key])
-                self.document.save()
-                messages.add_message(self.request, messages.INFO, 'Your new document has been added and saved.')
-
+            self.form = self.process_post_form('Your new document has been added and saved.')
+        else:
+            self.form = document_detail_form_factory(form=self.form, document_type=self.document_type)
         return self.form
 
 
